import { loadImages, mkTexture } from 'util/image';
import { mapValues, nextPowerOf2 } from '../util';

import { TileCoords } from './tile';

import MapState from '../model/state/MapState';
import MapView from './MapView';
import Renderer from './renderer/Renderer';

export interface TileMap {
	texture: WebGLTexture;
	tiles: Map<string, TileCoords>;
}

function loadTiles(tiles: Map<string, string>): Promise<Map<string, HTMLImageElement>> {
	return loadImages(mapValues((t) => `resources/sprite/tile/${t}.png`, tiles));
}

function mkTileMap(
	gl: WebGLRenderingContext,
	tiles: Map<string, HTMLImageElement>,
): TileMap {
	const tileSize = 32;

	const canvasDim = nextPowerOf2(Math.sqrt(tiles.size));
	const canvasSize = canvasDim * tileSize;

	const canvas = document.createElement('canvas');
	canvas.width = canvas.height = canvasSize;

	let x = 0, y = 0;
	const map: Map<string, TileCoords> = new Map();
	const ctx = canvas.getContext('2d') as CanvasRenderingContext2D;

	for (const [k, tile] of tiles) {
		ctx.drawImage(tile, x * tileSize, y * tileSize);
		map.set(k, [x / canvasDim, y / canvasDim, (x + 1) / canvasDim, (y + 1) / canvasDim]);

		x++;
		if (x === canvasDim) {
			x = 0;
			y++;
		}
	}

	return {
		texture: mkTexture(gl, canvas),
		tiles: map,
	};
}

export async function loadMap(r: Renderer, map: MapState): Promise<MapView> {
	const tiles = await loadTiles(map.data.tiles);
	const tileMap = mkTileMap(r.getContext(), tiles);

	return new MapView(r, map, tileMap);
}
